# Copyright (c) 2021, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from gzip import GzipFile
from io import BytesIO
try:
    from urllib.request import urlopen
    from urllib.error import URLError
    import urllib.request as request
except ImportError:
    from urllib2 import urlopen
    from urllib2 import URLError
    import urllib2 as request

from ._version import __version__


def urlopen_gzip(url, **kwargs):
    # http/https URLs need custom requests to specify the user-agent, since some repositories reject
    # requests from the default user-agent.
    if url.startswith('http://') or url.startswith('https://'):
        url_request = request.Request(url, headers={
            'Accept-Encoding': 'gzip',
            'User-Agent': 'rosdep/{version}'.format(version=__version__),
        })
        response = urlopen(url_request, **kwargs)
        if response.info().get('Content-Encoding') == 'gzip':
            buffer = BytesIO(response.read())
            return GzipFile(fileobj=buffer, mode='rb')
        return response

    return urlopen(url, **kwargs)



# Copyright 2018 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys

import pytest


@pytest.mark.flake8
@pytest.mark.linter
def test_flake8():
    # flake8 doesn't have a stable public API as of ver 6.1.0.
    # See: https://flake8.pycqa.org/en/latest/user/python-api.html
    # Calling through subprocess is the most stable way to run it.

    result = subprocess.run(
        [sys.executable, '-m', 'flake8'],
        cwd=os.path.dirname(os.path.dirname(__file__)),
        check=False,
    )
    assert 0 == result.returncode, 'flake8 found violations'



import os
import subprocess as sp
import tempfile
import unittest

from rosdep2 import main as rdmain


class Issue30TestCase(unittest.TestCase):

    @unittest.skip('Issue is not resolved, see ros-infrastructure/rosdep#37')
    def testIssue30(self):
        d = make_temp_dir()
        try:
            cd(d)
            script = """#!/bin/bash
mkdir ws
mkdir ws/src
mkdir ws/build
cd ws/src
catkin_init_workspace > /dev/null
cd ..
cd build
cmake ../src > /dev/null
cd ../src
git clone -b groovy-devel git://github.com/ros-drivers/joystick_drivers.git > /dev/null
"""
            run_script('commands.bash', script)
            source('ws/build/devel/setup.sh')
            lookup = make_lookup()
            keys = rdmain.get_keys(lookup, ['spacenav_node'], recursive=True)
            expected = """
                geometry_msgs
                libspnav-dev
                libx11-dev
                roscpp
                sensor_msgs
                spacenavd
            """.split()
            self.assertEqual(expected, sorted(keys))
        finally:
            sp.call(['rm', '-rf', d])


def make_temp_dir():
    return tempfile.mkdtemp()


def cd(d):
    os.chdir(d)


def write_file(filename, contents):
    with open(filename, 'w') as f:
        f.write(contents)


def run_script(filename, contents):
    """
    Creates a script with the given filename and contents, runs it
    and returns the output.
    """
    write_file(filename, contents)
    os.chmod(filename, 0x755)
    p = sp.Popen([os.path.join('.', filename)], stdout=sp.PIPE)
    p.wait()
    return p.stdout.read()

# http://pythonwise.blogspot.com/2010/04/sourcing-shell-script.html


def source(script):
    """
    Sources a shell script at a given path, updating the environment.
    """
    pipe = sp.Popen('. %s; env' % script, stdout=sp.PIPE, shell=True)
    data = pipe.communicate()[0]
    env = dict((line.split('=', 1) for line in data.splitlines()))
    os.environ.update(env)


def make_lookup():
    """
    Creates a RosdepLookup object.
    """
    import rosdep2.sources_list as sl
    import rosdep2.lookup as rdl
    sources_loader = sl.SourcesListLoader.create_default(
        sources_cache_dir=sl.get_sources_cache_dir(),
        verbose=True)
    return rdl.RosdepLookup.create_from_rospkg(sources_loader=sources_loader)


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Tully Foote/tfoote@willowgarage.com

import os
import subprocess
import sys
import re

from configparser import ConfigParser
from pathlib import Path

try:
    import importlib.metadata as importlib_metadata
except ImportError:
    import importlib_metadata

from packaging.requirements import Requirement
from packaging.version import InvalidVersion, parse
from ..core import InstallFailed
from ..installers import PackageManagerInstaller
from ..shell_utils import read_stdout

# pip package manager key
PIP_INSTALLER = 'pip'

EXTERNALLY_MANAGED_EXPLAINER = """
rosdep installation of pip packages requires installing packages globally as root.
When using Python >= 3.11, PEP 668 compliance requires you to allow pip to install alongside
externally managed packages using the 'break-system-packages' option.
The recommeded way to set this option when using rosdep is to set the environment variable
PIP_BREAK_SYSTEM_PACKAGES=1
in your environment.

For more information refer to http://docs.ros.org/en/independent/api/rosdep/html/pip_and_pep_668.html
"""


def register_installers(context):
    context.set_installer(PIP_INSTALLER, PipInstaller())


def get_pip_command():
    # First try user specified pip cmd
    if 'ROSDEP_PIP_CMD' in os.environ:
        cmd = os.environ['ROSDEP_PIP_CMD'].split()
        if is_cmd_available(cmd + ['--help']):
            return cmd
        return None

    # next try pip2 or pip3
    cmd = ['pip' + os.environ['ROS_PYTHON_VERSION']]
    if is_cmd_available(cmd):
        return cmd

    # Second, try using the same python executable since we know that exists
    if os.environ['ROS_PYTHON_VERSION'] == sys.version[0]:
        try:
            import pip
        except ImportError:
            pass
        else:
            return [sys.executable, '-m', 'pip']

    # Finally, try python2 or python3 commands
    cmd = ['python' + os.environ['ROS_PYTHON_VERSION'], '-m', 'pip']
    if is_cmd_available(cmd):
        return cmd
    return None


def in_virtual_environment():
    return sys.prefix != sys.base_prefix


def externally_managed_installable():
    """
    PEP 668 enacted in Python 3.11 blocks pip from working in "externally
    managed" environments such as operating systems with included package
    managers. If we're on Python 3.11 or greater, we need to check that pip
    is configured to allow installing system-wide packages with the
    flagrantly named "break system packages" config option or environment
    variable.
    """

    # This doesn't affect Python versions before 3.11
    if sys.version_info < (3, 11):
        return True

    # This doesn't affect Python virtual environments
    if in_virtual_environment():
        return True

    if (
            'PIP_BREAK_SYSTEM_PACKAGES' in os.environ and
            os.environ['PIP_BREAK_SYSTEM_PACKAGES'].lower() in ('yes', '1', 'true')
    ):
        return True

    # Check the same configuration directories as pip does per
    # https://pip.pypa.io/en/stable/topics/configuration/
    pip_config = ConfigParser()
    if 'XDG_CONFIG_DIRS' in os.environ:
        for xdg_dir in os.environ['XDG_CONFIG_DIRS'].split(':'):
            pip_config_file = Path(xdg_dir) / 'pip' / 'pip.conf'
            pip_config.read(pip_config_file)
            if pip_config.getboolean('install', 'break-system-packages', fallback=False):
                return True

    fallback_config = Path('/etc/pip.conf')
    pip_config.read(fallback_config)
    if pip_config.getboolean('install', 'break-system-packages', fallback=False):
        return True
    # On Python 3.11 and later, when no explicit configuration is present,
    # global pip installation will not work.
    return False


def is_cmd_available(cmd):
    try:
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        _ = proc.communicate()
        return 0 == proc.returncode
    except OSError:
        return False


def parse_version(version_str):
    """
    Given a textual representation of a Python package version, return its parsed representation.

    :param version_str: The textual representation of the version.
    :return: The parsed representation of None if the version cannot be parsed.
    :rtype: packaging.version.Version or packaging.version.LegacyVersion or None
    """
    try:
        return parse(version_str)
    except InvalidVersion:
        return None


def pip_detect(pkgs, exec_fn=None):
    """
    Given a list of package specifications, return the list of installed
    packages which meet the specifications.

    :param exec_fn: function to execute Popen and read stdout (for testing)
    """
    pip_cmd = get_pip_command()
    if not pip_cmd:
        return []

    if exec_fn is None:
        exec_fn = read_stdout
    pkg_list = exec_fn(pip_cmd + ['freeze']).split('\n')
    pkg_list = [p for p in pkg_list if len(p) > 0]

    ret_list = []
    version_list = []
    req_list = []

    for pkg in pkg_list:
        pkg_row = pkg.split('==')

        # skip over locally editable packages
        line = pkg_row[0].strip()
        if line.startswith("#") or line.startswith('-e'):
            continue

        # skip over other errors
        if len(pkg_row) != 2:
            print(f"Incomprehensibled package line: {pkg}")
            continue

        # Account for some unusual instances of === instead of ==
        pkg_row[1] = pkg_row[1].strip('=')

        version_list.append((pkg_row[0], parse_version(pkg_row[1])))

    for pkg in pkgs:
        req_list.append(Requirement(pkg))

    def canonicalize_name(name):
        return re.sub(r"[-_.]+", "-", name).lower()

    if (os.environ.get('ROSDEP_DEBUG') or '').lower() in ('yes', '1', 'true'):
        print("REQ:", req_list)
        print("PKG:", pkgs)
        print("VER:", version_list)
        print("PKGLST:", pkg_list)

    for req in req_list:
        for pkg in [ver for ver in version_list if canonicalize_name(ver[0]) == canonicalize_name(req.name)]:
            if pkg[1] is None or pkg[1] in req.specifier:
                ret_list.append(req.name)

    return ret_list


class PipInstaller(PackageManagerInstaller):
    """
    :class:`Installer` support for pip.
    """

    def __init__(self):
        super(PipInstaller, self).__init__(pip_detect, supports_depends=True)

        # Pass necessary environment for pip functionality via sudo
        if self.as_root and self.sudo_command != '':
            self.sudo_command += ' --preserve-env=PIP_BREAK_SYSTEM_PACKAGES'

    def get_version_strings(self):
        pip_version = importlib_metadata.version('pip')
        # keeping the name "setuptools" for backward compatibility
        setuptools_version = importlib_metadata.version('setuptools')
        version_strings = [
            'pip {}'.format(pip_version),
            'setuptools {}'.format(setuptools_version),
        ]
        return version_strings

    def get_packages_to_install(self, resolved, reinstall=False):
        if reinstall:
            return resolved
        if not resolved:
            return []
        else:
            detected = self.detect_fn(resolved)
            return [x for x in resolved if Requirement(x).name not in detected]

    def get_install_command(self, packages, interactive=True, reinstall=False, quiet=False, oneshot=[]):
        pip_cmd = get_pip_command()
        if not pip_cmd:
            raise InstallFailed((PIP_INSTALLER, 'pip is not installed'))
        if not externally_managed_installable():
            raise InstallFailed((PIP_INSTALLER, EXTERNALLY_MANAGED_EXPLAINER))
        if not packages:
            return []
        cmd = pip_cmd + ['install', '-U']
        if quiet:
            cmd.append('-q')
        if reinstall:
            cmd.append('-I')
        if 'pip' in oneshot:
            return [self.elevate_priv(cmd + sorted(packages))]
        return [self.elevate_priv(cmd + [p]) for p in packages]



# Copyright (c) 2011, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Ken Conley/kwc@willowgarage.com

import os
import sys
import traceback
from unittest.mock import Mock, patch


def get_test_dir():
    return os.path.abspath(os.path.join(os.path.dirname(__file__), 'pip'))


def test_pip_detect():
    from rosdep2.platforms.pip import pip_detect

    m = Mock()

    # test behavior with empty freeze
    m.return_value = ''
    val = pip_detect([], exec_fn=m)
    assert val == [], val

    val = pip_detect(['paramiko'], exec_fn=m)
    assert val == [], val

    # read freeze output into mock exec_fn
    with open(os.path.join(get_test_dir(), 'freeze_output'), 'r') as f:
        m.return_value = f.read()
    val = pip_detect(['paramiko'], exec_fn=m)
    assert val == ['paramiko'], val

    val = pip_detect(['paramiko', 'fakito', 'pycrypto'], exec_fn=m)
    assert val == ['paramiko', 'pycrypto'], val

    val = pip_detect(['paramiko', 'Brlapi==0.5.4', 'pycrypto'], exec_fn=m)
    assert val == ['paramiko', 'Brlapi', 'pycrypto'], val

    val = pip_detect(['paramiko', 'Brlapi==0.5.5', 'pycrypto'], exec_fn=m)
    assert val == ['paramiko', 'pycrypto'], val

    val = pip_detect(['Brlapi>0.5'], exec_fn=m)
    assert val == ['Brlapi'], val

    val = pip_detect(['Brlapi>0.5,<0.6'], exec_fn=m)
    assert val == ['Brlapi'], val

    val = pip_detect(['Brlapi>0.5,<0.5.4'], exec_fn=m)
    assert val == [], val


def test_PipInstaller_get_depends():
    # make sure PipInstaller supports depends
    from rosdep2.platforms.pip import PipInstaller
    installer = PipInstaller()
    assert ['foo'] == installer.get_depends(dict(depends=['foo']))


@patch('rosdep2.platforms.pip.externally_managed_installable')
def test_PipInstaller_handles_externally_managed_environment(externally_managed_installable):
    from rosdep2 import InstallFailed
    from rosdep2.platforms.pip import EXTERNALLY_MANAGED_EXPLAINER, PipInstaller

    externally_managed_installable.return_value = False
    installer = PipInstaller()
    try:
        installer.get_install_command(['whatever'])
        assert False, 'should have raised'
    except InstallFailed as e:
        assert e.failures == [('pip', EXTERNALLY_MANAGED_EXPLAINER)]
    externally_managed_installable.return_value = True
    assert installer.get_install_command(['whatever'], interactive=False)


@patch('rosdep2.platforms.pip.sys.version_info', new=(3, 11))
@patch('rosdep2.platforms.pip.sys.prefix', sys.base_prefix)
@patch.dict(os.environ, {'PIP_BREAK_SYSTEM_PACKAGES': '0'})
def test_externally_managed_installable():
    from rosdep2.platforms.pip import externally_managed_installable
    assert externally_managed_installable() is False

    @patch('rosdep2.platforms.pip.sys.version_info', new=(3, 10))
    def test_last_exempt_version():
        assert externally_managed_installable()

    @patch('rosdep2.platforms.pip.sys.base_prefix', new='/some/other/path')
    def test_virtual_environment():
        assert externally_managed_installable()

    @patch.dict(os.environ, {'PIP_BREAK_SYSTEM_PACKAGES': '1'})
    def test_break_system_packages_env_var():
        assert externally_managed_installable()

    from configparser import ConfigParser
    from pathlib import Path

    @patch.object(ConfigParser, 'read')
    @patch.object(ConfigParser, 'getboolean')
    @patch.dict(os.environ, {'XDG_CONFIG_DIRS': '/xdg'})
    def test_xdg_pip_dot_conf(getboolean, read):
        getboolean.return_value = True
        assert externally_managed_installable()
        read.assert_called_once_with(Path('/xdg/pip/pip.conf'))
        getboolean.assert_called_once_with('install', 'break-system-packages', fallback=False)

    @patch.object(ConfigParser, 'read')
    @patch.object(ConfigParser, 'getboolean')
    def test_pip_dot_conf(getboolean, read):
        getboolean.return_value = True
        assert externally_managed_installable()
        read.assert_called_once_with(Path('/etc/pip.conf'))
        getboolean.assert_called_once_with('install', 'break-system-packages', fallback=False)

    test_last_exempt_version()
    test_virtual_environment()
    test_break_system_packages_env_var()
    test_xdg_pip_dot_conf()
    test_pip_dot_conf()


@patch.dict(os.environ, {'PIP_BREAK_SYSTEM_PACKAGES': '1'})
def test_PipInstaller():
    from rosdep2 import InstallFailed
    from rosdep2.platforms.pip import PipInstaller

    @patch('rosdep2.platforms.pip.get_pip_command')
    def test_no_pip(mock_method):
        mock_method.return_value = None
        try:
            installer = PipInstaller()
            installer.get_install_command(['whatever'])
            assert False, 'should have raised'
        except InstallFailed:
            pass

    test_no_pip()

    @patch('rosdep2.platforms.pip.get_pip_command')
    @patch.object(PipInstaller, 'get_packages_to_install')
    def test(expected_prefix, mock_method, mock_get_pip_command):
        mock_get_pip_command.return_value = ['mock-pip']
        installer = PipInstaller()

        # no interactive option with PIP
        mock_method.return_value = ['a', 'b']

        expected = [expected_prefix + ['mock-pip', 'install', '-U', 'whatever']]
        val = installer.get_install_command(['whatever'], interactive=False)
        assert val == expected, val
        expected = [expected_prefix + ['mock-pip', 'install', '-U', 'whatever']]
        val = installer.get_install_command(['whatever'], interactive=True)
        assert val == expected, val

        # oneshot
        expected = [expected_prefix + ['mock-pip', 'install', '-U', 'whatever']]
        val = installer.get_install_command(['whatever'], interactive=False, oneshot=['pip'])
        assert val == expected, val
        expected = [expected_prefix + ['mock-pip', 'install', '-U', 'whatever']]
        val = installer.get_install_command(['whatever'], interactive=True, oneshot=['pip'])
        assert val == expected, val
    try:
        if hasattr(os, 'geteuid'):
            with patch('rosdep2.installers.os.geteuid', return_value=1):
                test(['sudo', '-H', '--preserve-env=PIP_BREAK_SYSTEM_PACKAGES'])
            with patch('rosdep2.installers.os.geteuid', return_value=0):
                test([])
        else:
            test([])
    except AssertionError:
        traceback.print_exc()
        raise


def test_PipInstaller_get_version_strings():
    from rosdep2.platforms.pip import PipInstaller

    installer = PipInstaller()
    assert installer.get_version_strings()


def test_get_pip_command():
    from rosdep2.platforms.pip import get_pip_command

    # pip2 or pip3
    @patch('rosdep2.platforms.pip.is_cmd_available')
    def test_pip2_or_pip3(mock_is_cmd_available):
        mock_is_cmd_available.return_value = True

        with patch.dict(os.environ, {'ROS_PYTHON_VERSION': '2'}):
            assert ['pip2'] == get_pip_command()

        with patch.dict(os.environ, {'ROS_PYTHON_VERSION': '3'}):
            assert ['pip3'] == get_pip_command()

    # sys.executable (assumes pip is installed)
    @patch('rosdep2.platforms.pip.is_cmd_available')
    def test_sys_executable(mock_is_cmd_available):
        mock_is_cmd_available.return_value = False

        with patch.dict(os.environ, {'ROS_PYTHON_VERSION': str(sys.version[0])}):
            assert [sys.executable, '-m', 'pip'] == get_pip_command()

    # python2 or python3
    def fake_is_cmd_available(cmd):
        if cmd[0] in ['pip2', 'pip3']:
            return False
        return True

    @patch('rosdep2.platforms.pip.sys.executable', 'python3')
    @patch('rosdep2.platforms.pip.is_cmd_available', new=fake_is_cmd_available)
    def test_python2_or_python3():

        with patch.dict(os.environ, {'ROS_PYTHON_VERSION': '2'}):
            assert ['python2', '-m', 'pip'] == get_pip_command()

        with patch.dict(os.environ, {'ROS_PYTHON_VERSION': '3'}):
            assert ['python3', '-m', 'pip'] == get_pip_command()

    def fake_is_uv_available(cmd):
        assert cmd == ['uv', 'pip', '--help']
        return cmd[0] in ['uv']

    @patch('rosdep2.platforms.pip.is_cmd_available', new=fake_is_uv_available)
    def test_uv():
        with patch.dict(os.environ, {'ROSDEP_PIP_CMD': 'uv pip'}):
            assert ['uv', 'pip'] == get_pip_command()

    test_uv()
    test_pip2_or_pip3()
    test_sys_executable()
    test_python2_or_python3()



# Copyright (c) 2011, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os


def test_RosdepInternalError():
    from rosdep2.core import RosdepInternalError
    try:
        raise Exception('foo')
    except Exception as e:
        ex = RosdepInternalError(e)
        assert e == ex.error


def test_rd_debug():
    # just tripwire/coverage
    from rosdep2.core import rd_debug
    rd_debug('foo')
    os.environ['ROSDEP_DEBUG'] = '1'
    rd_debug('foo')


def test_InvalidData():
    from rosdep2.core import InvalidData
    try:
        raise InvalidData('hi')
    except InvalidData as ex:
        assert 'hi' in str(ex)



# Copyright (c) 2011, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Ken Conley/kwc@willowgarage.com

import os
import traceback
from unittest.mock import patch, Mock


def get_test_dir():
    # not used yet
    return os.path.abspath(os.path.join(os.path.dirname(__file__), 'redhat'))


def test_rpm_expand_cmd():
    from rosdep2.platforms.redhat import rpm_expand_cmd

    m = Mock()
    m.return_value = ''

    # Non-macro test, should return the string unmodified
    val = rpm_expand_cmd('test-string', exec_fn=m)
    assert val == 'test-string', val

    # Macro test, should return expanded rpm tag
    with open(os.path.join(get_test_dir(), 'rpm-E-fedora'), 'r') as f:
        m.return_value = f.read()
    val = rpm_expand_cmd('%fedora', exec_fn=m)
    assert val == '27', val


def test_rpm_detect():
    from rosdep2.platforms.redhat import rpm_detect

    m = Mock()
    m.return_value = ''

    val = rpm_detect([], exec_fn=m)
    assert val == [], val

    # Negitive case test. rpms use devel, rather than dev
    with open(os.path.join(get_test_dir(), 'rpm-q-tinyxml-dev'), 'r') as f:
        m.return_value = f.read()
    val = rpm_detect(['tinyxml-dev'], exec_fn=m)
    assert val == [], val

    # Positive case test. rpm should always be installed if you're attempting to detect rpms
    with open(os.path.join(get_test_dir(), 'rpm-q-rpm'), 'r') as f:
        m.return_value = f.read()
    val = rpm_detect(['rpm'], exec_fn=m)
    assert val == ['rpm'], val


def test_DnfInstaller():
    from rosdep2.platforms.redhat import DnfInstaller

    @patch.object(DnfInstaller, 'get_packages_to_install')
    def test(expected_prefix, mock_method):
        installer = DnfInstaller()
        mock_method.return_value = []
        assert [] == installer.get_install_command(['fake'])

        # no interactive option with YUM
        mock_method.return_value = ['a', 'b']
        expected = [expected_prefix + ['dnf', '--assumeyes', '--quiet', '--setopt=strict=0', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=False, quiet=True)
        assert val == expected, val + expected
        expected = [expected_prefix + ['dnf', '--quiet', '--setopt=strict=0', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=True, quiet=True)
        assert val == expected, val + expected
        expected = [expected_prefix + ['dnf', '--assumeyes', '--setopt=strict=0', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=False, quiet=False)
        assert val == expected, val + expected
        expected = [expected_prefix + ['dnf', '--setopt=strict=0', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=True, quiet=False)
        assert val == expected, val + expected
    try:
        if hasattr(os, 'geteuid'):
            with patch('rosdep2.installers.os.geteuid', return_value=1):
                test(['sudo', '-H'])
            with patch('rosdep2.installers.os.geteuid', return_value=0):
                test([])
        else:
            test([])
    except AssertionError:
        traceback.print_exc()
        raise


def test_YumInstaller():
    from rosdep2.platforms.redhat import YumInstaller

    @patch.object(YumInstaller, 'get_packages_to_install')
    def test(expected_prefix, mock_method):
        installer = YumInstaller()
        mock_method.return_value = []
        assert [] == installer.get_install_command(['fake'])

        # no interactive option with YUM
        mock_method.return_value = ['a', 'b']
        expected = [expected_prefix + ['yum', '--assumeyes', '--quiet', '--skip-broken', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=False, quiet=True)
        assert val == expected, val + expected
        expected = [expected_prefix + ['yum', '--quiet', '--skip-broken', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=True, quiet=True)
        assert val == expected, val + expected
        expected = [expected_prefix + ['yum', '--assumeyes', '--skip-broken', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=False, quiet=False)
        assert val == expected, val + expected
        expected = [expected_prefix + ['yum', '--skip-broken', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=True, quiet=False)
        assert val == expected, val + expected
    try:
        if hasattr(os, 'geteuid'):
            with patch('rosdep2.installers.os.geteuid', return_value=1):
                test(['sudo', '-H'])
            with patch('rosdep2.installers.os.geteuid', return_value=0):
                test([])
        else:
            test([])
    except AssertionError:
        traceback.print_exc()
        raise


def test_Fedora_variable_installer_key():
    from rosdep2 import InstallerContext
    from rosdep2.platforms import pip, redhat, source
    from rosdep2.platforms.redhat import DNF_INSTALLER, YUM_INSTALLER

    from rospkg.os_detect import OsDetect, OS_FEDORA
    os_detect_mock = Mock(spec=OsDetect)
    os_detect_mock.get_name.return_value = 'fedora'
    os_detect_mock.get_codename.return_value = 'twenty'
    os_detect_mock.get_version.return_value = '21'

    # create our test fixture.  use most of the default toolchain, but
    # replace the apt installer with one that we can have more fun
    # with.  we will do all tests with ubuntu lucid keys -- other
    # tests should cover different resolution cases.
    context = InstallerContext(os_detect_mock)
    pip.register_installers(context)
    redhat.register_installers(context)
    source.register_installers(context)
    redhat.register_platforms(context)

    assert YUM_INSTALLER == context.get_default_os_installer_key(OS_FEDORA)

    os_detect_mock.get_version.return_value = '22'
    assert DNF_INSTALLER == context.get_default_os_installer_key(OS_FEDORA)


def test_Fedora_variable_lookup_key():
    from rosdep2 import InstallerContext
    from rosdep2.platforms import pip, redhat, source
    from rosdep2.platforms.redhat import DNF_INSTALLER, YUM_INSTALLER

    from rospkg.os_detect import OsDetect, OS_FEDORA
    os_detect_mock = Mock(spec=OsDetect)
    os_detect_mock.get_name.return_value = 'fedora'
    os_detect_mock.get_codename.return_value = 'heisenbug'
    os_detect_mock.get_version.return_value = '20'

    # create our test fixture.  use most of the default toolchain, but
    # replace the apt installer with one that we can have more fun
    # with.  we will do all tests with ubuntu lucid keys -- other
    # tests should cover different resolution cases.
    context = InstallerContext(os_detect_mock)
    pip.register_installers(context)
    redhat.register_installers(context)
    source.register_installers(context)
    redhat.register_platforms(context)

    assert ('fedora', 'heisenbug') == context.get_os_name_and_version()

    os_detect_mock.get_codename.return_value = 'twenty'
    os_detect_mock.get_version.return_value = '21'
    assert (OS_FEDORA, '21') == context.get_os_name_and_version()



# Copyright (c) 2012, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author William Woodall/wjwwood@gmail.com


def test_DependencyGraph_Linear():
    from rosdep2.dependency_graph import DependencyGraph
    # Normal A-B-C
    dg = DependencyGraph()
    dg['A']['installer_key'] = 'a_installer'
    dg['A']['install_keys'] = ['a']
    dg['A']['dependencies'] = ['B']
    dg['B']['installer_key'] = 'b_installer'
    dg['B']['install_keys'] = ['b']
    dg['B']['dependencies'] = ['C']
    dg['C']['installer_key'] = 'c_installer'
    dg['C']['install_keys'] = ['c']
    dg['C']['dependencies'] = []
    result = dg.get_ordered_dependency_list()
    expected = {'c_installer': ['c'], 'b_installer': ['b'], 'a_installer': ['a']}
    assert result == expected, 'Results did not match expectations: %s == %s' % (str(result), str(expected))


def test_DependencyGraph_Cycle():
    from rosdep2.dependency_graph import DependencyGraph
    # Full Loop A-B-C-A-...
    dg = DependencyGraph()
    dg['A']['installer_key'] = 'a_installer'
    dg['A']['install_keys'] = ['a']
    dg['A']['dependencies'] = ['B']
    dg['B']['installer_key'] = 'b_installer'
    dg['B']['install_keys'] = ['b']
    dg['B']['dependencies'] = ['C']
    dg['C']['installer_key'] = 'c_installer'
    dg['C']['install_keys'] = ['c']
    dg['C']['dependencies'] = ['A']
    try:
        result = dg.get_ordered_dependency_list()
        assert False, "Doesn't fail, it should fail with an AssertionError because of the cycle."
    except AssertionError as e:
        if not str(e).startswith('A cycle in the dependency graph occurred with key'):
            assert False, 'Throws AssertionError, but with the wrong message. Error was: %s: %s' % (type(e), str(e))
    except Exception as e:
        assert False, 'Throws and Exception, but not an AssertionError. Error was: %s: %s' % (type(e), str(e))


def test_DependencyGraph_Short_Cycle():
    from rosdep2.dependency_graph import DependencyGraph
    # Short cycle A-B-C-D-B-C-D-...
    dg = DependencyGraph()
    dg['A']['installer_key'] = 'a_installer'
    dg['A']['install_keys'] = ['a']
    dg['A']['dependencies'] = ['B']
    dg['B']['installer_key'] = 'b_installer'
    dg['B']['install_keys'] = ['b']
    dg['B']['dependencies'] = ['C']
    dg['C']['installer_key'] = 'c_installer'
    dg['C']['install_keys'] = ['c']
    dg['C']['dependencies'] = ['D']
    dg['D']['installer_key'] = 'd_installer'
    dg['D']['install_keys'] = ['d']
    dg['D']['dependencies'] = ['B']
    try:
        result = dg.get_ordered_dependency_list()
        assert False, "Doesn't fail, it should fail with an AssertionError because of the cycle."
    except AssertionError as e:
        if not str(e).startswith('A cycle in the dependency graph occurred with key'):
            assert False, 'Throws AssertionError, but with the wrong message. Error was: %s: %s' % (type(e), str(e))
    except Exception as e:
        assert False, 'Throws and Exception, but not an AssertionError. Error was: %s: %s' % (type(e), str(e))


def test_DependencyGraph_Invalid_Key():
    from rosdep2.dependency_graph import DependencyGraph
    # Invalid graph A-B-C where C doesn't exist
    dg = DependencyGraph()
    dg['A']['installer_key'] = 'a_installer'
    dg['A']['install_keys'] = ['a']
    dg['A']['dependencies'] = ['B']
    dg['B']['installer_key'] = 'b_installer'
    dg['B']['install_keys'] = ['b']
    dg['B']['dependencies'] = ['C']
    try:
        result = dg.get_ordered_dependency_list()
        assert False, "Doesn't fail, it should fail with an KeyError because of the invalid rosdep key."
    except KeyError as e:
        if not str(e).endswith("does not exist in the dictionary of resolutions.'"):
            assert False, 'Throws KeyError, but with the wrong message. Error was: %s: %s' % (type(e), str(e))
    except Exception as e:
        assert False, 'Throws and Exception, but not an KeyError. Error was: %s: %s' % (type(e), str(e))


def test_DependencyGraph_Invalid_Key2():
    from rosdep2.dependency_graph import DependencyGraph
    # Invalid graph A-B-C where B doesn't exist
    dg = DependencyGraph()
    dg['A']['installer_key'] = 'a_installer'
    dg['A']['install_keys'] = ['a']
    dg['A']['dependencies'] = ['B']
    dg['C']['installer_key'] = 'c_installer'
    dg['C']['install_keys'] = ['c']
    dg['C']['dependencies'] = []
    try:
        result = dg.get_ordered_dependency_list()
        assert False, "Doesn't fail, it should fail with an KeyError because of the invalid rosdep key."
    except KeyError as e:
        if not str(e).endswith("does not exist in the dictionary of resolutions.'"):
            assert False, 'Throws KeyError, but with the wrong message. Error was: %s: %s' % (type(e), str(e))
    except Exception as e:
        assert False, 'Throws and Exception, but not an KeyError. Error was: %s: %s' % (type(e), str(e))


def test_DependencyGraph_Multi_Root():
    from rosdep2.dependency_graph import DependencyGraph
    # Multi root, shared dependency: A-B-C, D-C
    dg = DependencyGraph()
    dg['A']['installer_key'] = 'a_installer'
    dg['A']['install_keys'] = ['a']
    dg['A']['dependencies'] = ['B']
    dg['B']['installer_key'] = 'b_installer'
    dg['B']['install_keys'] = ['b']
    dg['B']['dependencies'] = ['C']
    dg['C']['installer_key'] = 'c_installer'
    dg['C']['install_keys'] = ['c']
    dg['C']['dependencies'] = []
    dg['D']['installer_key'] = 'd_installer'
    dg['D']['install_keys'] = ['d']
    dg['D']['dependencies'] = ['C']
    result = dg.get_ordered_dependency_list()
    # TODO: The expected might also have a different order, for example it might be:
    # [('c_installer', ['c']), ('d_installer', ['d']), ('b_installer', ['b']), ('a_installer', ['a'])]
    # But that wont invalidate the order from a dependency graph stand point
    expected = [
        {'c_installer': ['c'], 'b_installer': ['b'], 'a_installer': ['a'], 'd_installer': ['d']},
        {'c_installer': ['c'], 'd_installer': ['d'], 'b_installer': ['b'], 'a_installer': ['a']},
    ]
    assert result in expected, 'Results did not match expectations: %s == %s' % (str(result), str(expected))


def test_DependencyGraph_Realworld():
    from rosdep2.dependency_graph import DependencyGraph
    # Real world example
    dg = DependencyGraph()
    dg['python-matplotlib']['installer_key'] = 'pip'
    dg['python-matplotlib']['install_keys'] = ['matplotlib']
    dg['python-matplotlib']['dependencies'] = ['pkg-config']
    dg['pkg-config']['installer_key'] = 'homebrew'
    dg['pkg-config']['install_keys'] = ['pkg-config']
    dg['pkg-config']['dependencies'] = []
    result = dg.get_ordered_dependency_list()
    expected = {'homebrew': ['pkg-config'], 'pip': ['matplotlib']}
    assert result == expected, 'Results did not match expectations: %s == %s' % (str(result), str(expected))



from rosdep2.catkin_support import get_installer, get_catkin_view, ValidationFailed, resolve_for_os

from rosdep2.platforms.debian import APT_INSTALLER
import pytest


@pytest.mark.usefixtures('fake_rosdep')
def test_workflow():
    installer = get_installer(APT_INSTALLER)
    view = get_catkin_view('fuerte', 'ubuntu', 'lucid')
    resolved = resolve_for_os('cmake', view, installer, 'ubuntu', 'lucid')
    assert ['cmake'] == resolved
    resolved = resolve_for_os('python', view, installer, 'ubuntu', 'lucid')
    assert resolved == ['python-dev']



# Copyright (c) 2011, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from contextlib import contextmanager
from unittest import SkipTest
from unittest.mock import Mock, patch
import os
import sys
try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

from rospkg import RosPack, RosStack


def get_test_dir():
    return os.path.abspath(os.path.dirname(__file__))


def get_cache_dir():
    p = os.path.join(get_test_dir(), 'sources_cache')
    assert os.path.isdir(p)
    return p


def create_test_SourcesListLoader():
    from rosdep2.sources_list import SourcesListLoader
    return SourcesListLoader.create_default(sources_cache_dir=get_cache_dir(), verbose=True)


def get_test_tree_dir():
    return os.path.abspath(os.path.join(os.path.dirname(__file__), 'tree'))


def get_test_rospkgs():
    test_dir = get_test_tree_dir()
    ros_root = os.path.join(test_dir, 'ros')
    ros_package_path = os.path.join(test_dir, 'stacks')
    ros_paths = [ros_root, ros_package_path]
    rospack = RosPack(ros_paths=ros_paths)
    rosstack = RosStack(ros_paths=ros_paths)
    return rospack, rosstack


def test_InstallerContext_ctor():
    from rosdep2.installers import InstallerContext
    from rospkg.os_detect import OsDetect

    context = InstallerContext()
    assert context.get_os_detect() is not None
    assert isinstance(context.get_os_detect(), OsDetect)

    detect = OsDetect()
    context = InstallerContext(detect)
    assert context.get_os_detect() == detect
    assert len(context.get_installer_keys()) == 0
    assert len(context.get_os_keys()) == 0

    context.verbose = True
    assert context.get_os_detect() == detect
    assert len(context.get_installer_keys()) == 0
    assert len(context.get_os_keys()) == 0


def test_InstallerContext_get_os_version_type():
    from rospkg.os_detect import OS_UBUNTU, OsDetect
    from rosdep2.installers import InstallerContext
    context = InstallerContext()

    try:
        context.set_os_version_type(OS_UBUNTU, 'bad')
        assert False, 'should check type'
    except ValueError:
        pass

    assert OsDetect.get_version == context.get_os_version_type(OS_UBUNTU)
    context.set_os_version_type(OS_UBUNTU, OsDetect.get_codename)
    assert OsDetect.get_codename == context.get_os_version_type(OS_UBUNTU)


def test_InstallerContext_os_version_and_name():
    from rospkg.os_detect import OsDetect
    from rosdep2.installers import InstallerContext
    context = InstallerContext()
    context.set_verbose(True)
    os_name, os_version = context.get_os_name_and_version()
    assert os_name is not None
    assert os_version is not None

    val = ('fakeos', 'blah')
    context.set_os_override(*val)
    assert val == context.get_os_name_and_version()

    os_detect_mock = Mock(spec=OsDetect)
    os_detect_mock.get_name.return_value = 'fakeos'
    os_detect_mock.get_version.return_value = 'fakeos-version'
    os_detect_mock.get_codename.return_value = 'fakeos-codename'
    context = InstallerContext(os_detect_mock)
    context.set_os_version_type('fakeos', os_detect_mock.get_codename)
    os_name, os_version = context.get_os_name_and_version()
    assert os_name == 'fakeos', os_name
    assert os_version == 'fakeos-codename', os_version

    context.set_os_version_type('fakeos', os_detect_mock.get_version)
    os_name, os_version = context.get_os_name_and_version()
    assert os_name == 'fakeos', os_name
    assert os_version == 'fakeos-version', os_version


def test_InstallerContext_installers():
    from rosdep2.installers import InstallerContext, Installer
    from rospkg.os_detect import OsDetect
    detect = OsDetect()
    context = InstallerContext(detect)
    context.verbose = True

    key = 'fake-apt'
    try:
        installer = context.get_installer(key)
        assert False, 'should have raised: %s' % (installer)
    except KeyError:
        pass

    class Foo:
        pass

    class FakeInstaller(Installer):
        pass

    class FakeInstaller2(Installer):
        pass

    # test TypeError on set_installer
    try:
        context.set_installer(key, 1)
        assert False, 'should have raised'
    except TypeError:
        pass
    try:
        context.set_installer(key, Foo())
        assert False, 'should have raised'
    except TypeError:
        pass
    try:
        # must be instantiated
        context.set_installer(key, FakeInstaller)
        assert False, 'should have raised'
    except TypeError:
        pass

    installer = FakeInstaller()
    installer2 = FakeInstaller2()
    context.set_installer(key, installer)
    assert context.get_installer(key) == installer
    assert list(context.get_installer_keys()) == [key]

    # repeat with same args
    context.set_installer(key, installer)
    assert context.get_installer(key) == installer
    assert list(context.get_installer_keys()) == [key]

    # repeat with new installer
    context.set_installer(key, installer2)
    assert context.get_installer(key) == installer2
    assert list(context.get_installer_keys()) == [key]

    # repeat with new key
    key2 = 'fake-port'
    context.set_installer(key2, installer2)
    assert context.get_installer(key2) == installer2
    assert set(context.get_installer_keys()) == set([key, key2])

    # test installer deletion
    key3 = 'fake3'
    context.set_installer(key3, installer2)
    assert context.get_installer(key3) == installer2
    assert set(context.get_installer_keys()) == set([key, key2, key3])
    context.set_installer(key3, None)
    try:
        context.get_installer(key3)
        assert False
    except KeyError:
        pass
    assert set(context.get_installer_keys()) == set([key, key2])


def test_InstallerContext_os_installers():
    from rosdep2.installers import InstallerContext, Installer
    from rospkg.os_detect import OsDetect
    detect = OsDetect()
    context = InstallerContext(detect)
    context.verbose = True

    os_key = 'ubuntu'
    try:
        context.get_os_installer_keys(os_key)
        assert False, 'should have raised'
    except KeyError:
        pass
    try:
        context.get_default_os_installer_key(os_key)
        assert False, 'should have raised'
    except KeyError:
        pass
    try:
        context.add_os_installer_key(os_key, 'fake-key')
        assert False, 'should have raised'
    except KeyError:
        pass
    try:
        context.set_default_os_installer_key(os_key, 'non-method')
        assert False, 'should have raised'
    except KeyError:
        pass
    try:
        context.set_default_os_installer_key(os_key, lambda self: 'fake-key')
        assert False, 'should have raised'
    except KeyError:
        pass
    try:
        context.get_default_os_installer_key('bad-os')
        assert False, 'should have raised'
    except KeyError:
        pass

    installer_key1 = 'fake1'
    installer_key2 = 'fake2'

    class FakeInstaller(Installer):
        pass

    class FakeInstaller2(Installer):
        pass

    # configure our context with two valid installers
    context.set_installer(installer_key1, FakeInstaller())
    context.set_installer(installer_key2, FakeInstaller2())

    # start adding installers for os_key
    context.add_os_installer_key(os_key, installer_key1)
    assert context.get_os_installer_keys(os_key) == [installer_key1]

    # retest set_default_os_installer_key, now with installer_key not configured on os
    try:
        context.set_default_os_installer_key(os_key, lambda self: installer_key2)
        assert False, 'should have raised'
    except KeyError as e:
        assert 'add_os_installer' in str(e), e

    # now properly add in key2
    context.add_os_installer_key(os_key, installer_key2)
    assert set(context.get_os_installer_keys(os_key)) == set([installer_key1, installer_key2])

    # test default
    assert context.get_default_os_installer_key(os_key) is None
    context.set_default_os_installer_key(os_key, lambda self: installer_key1)
    assert installer_key1 == context.get_default_os_installer_key(os_key)
    context.set_default_os_installer_key(os_key, lambda self: installer_key2)
    assert installer_key2 == context.get_default_os_installer_key(os_key)

    # retest set_default_os_installer_key, now with invalid os
    try:
        context.set_default_os_installer_key('bad-os', lambda self: installer_key1)
        assert False, 'should have raised'
    except KeyError:
        pass


def test_Installer_tripwire():
    from rosdep2.installers import Installer
    try:
        Installer().is_installed('foo')
        assert False
    except NotImplementedError:
        pass
    try:
        Installer().get_install_command('foo')
        assert False
    except NotImplementedError:
        pass
    try:
        Installer().resolve({})
        assert False
    except NotImplementedError:
        pass
    try:
        Installer().unique([])
        assert False
    except NotImplementedError:
        pass
    assert Installer().get_depends({}) == []


def detect_fn_empty(packages):
    return []


def detect_fn_all(packages):
    return packages
# return any packages that are string length 1


def detect_fn_single(packages):
    return [p for p in packages if len(p) == 1]


def test_PackageManagerInstaller():
    from rosdep2.installers import PackageManagerInstaller
    try:
        PackageManagerInstaller(detect_fn_all).get_install_command(['foo'])
        assert False
    except NotImplementedError:
        pass


def test_PackageManagerInstaller_resolve():
    from rosdep2 import InvalidData
    from rosdep2.installers import PackageManagerInstaller

    installer = PackageManagerInstaller(detect_fn_all)
    assert ['baz'] == installer.resolve(dict(depends=['foo', 'bar'], packages=['baz']))
    assert ['baz', 'bar'] == installer.resolve(dict(packages=['baz', 'bar']))

    # test string logic
    assert ['baz'] == installer.resolve(dict(depends=['foo', 'bar'], packages='baz'))
    assert ['baz', 'bar'] == installer.resolve(dict(packages='baz bar'))
    assert ['baz'] == installer.resolve('baz')
    assert ['baz', 'bar'] == installer.resolve('baz bar')

    # test list logic
    assert ['baz'] == installer.resolve(['baz'])
    assert ['baz', 'bar'] == installer.resolve(['baz', 'bar'])

    # test invalid data
    try:
        installer.resolve(0)
        assert False, 'should have raised'
    except InvalidData:
        pass


def test_PackageManagerInstaller_depends():
    from rosdep2.installers import PackageManagerInstaller

    installer = PackageManagerInstaller(detect_fn_all, supports_depends=True)
    assert ['foo', 'bar'] == installer.get_depends(dict(depends=['foo', 'bar'], packages=['baz']))
    installer = PackageManagerInstaller(detect_fn_all, supports_depends=False)
    assert [] == installer.get_depends(dict(depends=['foo', 'bar'], packages=['baz']))


def test_PackageManagerInstaller_unique():
    from rosdep2.installers import PackageManagerInstaller

    installer = PackageManagerInstaller(detect_fn_all)

    assert [] == installer.unique()
    assert [] == installer.unique([])
    assert [] == installer.unique([], [])
    assert ['a'] == installer.unique([], [], ['a'])
    assert ['a'] == installer.unique(['a'], [], ['a'])
    assert set(['a', 'b', 'c']) == set(installer.unique(['a', 'b', 'c'], ['a', 'b', 'c']))
    assert set(['a', 'b', 'c']) == set(installer.unique(['a'], ['b'], ['c']))
    assert set(['a', 'b', 'c']) == set(installer.unique(['a', 'b'], ['c']))
    assert set(['a', 'b', 'c']) == set(installer.unique(['a', 'b'], ['c', 'a']))


def test_PackageManagerInstaller_is_installed():
    from rosdep2.installers import PackageManagerInstaller

    installer = PackageManagerInstaller(detect_fn_all)
    for r in ['a', 'b', 'c']:
        assert installer.is_installed(r), installer.is_installed(r)
    installer = PackageManagerInstaller(detect_fn_empty)
    for r in ['a', 'b', 'c']:
        assert not installer.is_installed(r), installer.is_installed(r)


def test_PackageManagerInstaller_get_packages_to_install():
    from rosdep2.installers import PackageManagerInstaller

    installer = PackageManagerInstaller(detect_fn_all)
    assert [] == installer.get_packages_to_install([])
    assert [] == installer.get_packages_to_install(['a', 'b', 'c'])
    assert set(['a', 'b', 'c']) == set(installer.get_packages_to_install(['a', 'b', 'c'], reinstall=True))

    installer = PackageManagerInstaller(detect_fn_empty)
    assert set(['a', 'b', 'c']) == set(installer.get_packages_to_install(['a', 'b', 'c']))
    assert set(['a', 'b', 'c']) == set(installer.get_packages_to_install(['a', 'b', 'c'], reinstall=True))
    installer = PackageManagerInstaller(detect_fn_single)
    assert set(['baba', 'cada']) == set(installer.get_packages_to_install(['a', 'baba', 'b', 'cada', 'c']))


def test_RosdepInstaller_ctor():
    # tripwire/coverage
    from rosdep2 import create_default_installer_context
    from rosdep2.lookup import RosdepLookup
    from rosdep2.installers import RosdepInstaller
    context = create_default_installer_context()
    installer = RosdepInstaller(context)
    assert context == installer.installer_context


def test_RosdepInstaller_get_uninstalled():
    from rosdep2 import create_default_installer_context
    from rosdep2.lookup import RosdepLookup
    from rosdep2.installers import RosdepInstaller
    from rosdep2.platforms.debian import APT_INSTALLER

    rospack, rosstack = get_test_rospkgs()

    # create our test fixture.  use most of the default toolchain, but
    # replace the apt installer with one that we can have more fun
    # with.  we will do all tests with ubuntu lucid keys -- other
    # tests should cover different resolution cases.
    sources_loader = create_test_SourcesListLoader()
    lookup = RosdepLookup.create_from_rospkg(rospack=rospack, rosstack=rosstack, sources_loader=sources_loader)
    context = create_default_installer_context()
    context.set_os_override('ubuntu', 'lucid')
    installer = RosdepInstaller(context)

    # in this first test, detect_fn detects everything as installed
    fake_apt = get_fake_apt(lambda x: x)
    context.set_installer(APT_INSTALLER, fake_apt)

    for verbose in [True, False]:
        tests = [['roscpp_fake'], ['roscpp_fake', 'rospack_fake'], ['empty_package'],
                 ['roscpp_fake', 'rospack_fake', 'empty_package'],
                 ['roscpp_fake', 'rospack_fake'],
                 ]
        for test in tests:
            resolutions, errors = lookup.resolve_all(test, context)
            uninstalled = installer.get_uninstalled(resolutions, verbose)
            assert not uninstalled, uninstalled
            assert not errors, errors

    # in this second test, detect_fn detects nothing as installed
    fake_apt = get_fake_apt(lambda x: [])
    context.set_installer(APT_INSTALLER, fake_apt)

    for verbose in [True, False]:
        resolutions, errors = lookup.resolve_all(['empty'], context)
        uninstalled = installer.get_uninstalled(resolutions, verbose)
        assert not uninstalled, uninstalled
        assert not errors

        expected = set(['libltdl-dev', 'libboost1.40-all-dev', 'libtool'])
        resolutions, errors = lookup.resolve_all(['roscpp_fake'], context)
        uninstalled = installer.get_uninstalled(resolutions, verbose)
        apt_uninstalled = []
        for k, v in uninstalled.items():
            if k == APT_INSTALLER:
                apt_uninstalled.extend(v)
        assert list(set(uninstalled.keys())) == [APT_INSTALLER]
        assert set(apt_uninstalled) == expected
        assert not errors

        expected = ['libtinyxml-dev']
        resolutions, errors = lookup.resolve_all(['rospack_fake'], context)
        uninstalled = installer.get_uninstalled(resolutions, verbose)
        apt_uninstalled = []
        for k, v in uninstalled.items():
            if k == APT_INSTALLER:
                apt_uninstalled.extend(v)
        assert list(set(uninstalled.keys())) == [APT_INSTALLER]
        assert apt_uninstalled == expected, uninstalled
        assert not errors


def get_fake_apt(detect_fn):
    # mainly did this to keep coverage results
    from rosdep2.installers import PackageManagerInstaller

    class FakeAptInstaller(PackageManagerInstaller):
        """
        An implementation of the Installer for use on debian style
        systems.
        """
        def __init__(self):
            super(FakeAptInstaller, self).__init__(detect_fn)

        def get_install_command(self, resolved, interactive=True, reinstall=False):
            return [[resolved, interactive, reinstall]]
    return FakeAptInstaller()


def test_RosdepInstaller_get_uninstalled_unconfigured():
    from rosdep2 import create_default_installer_context, RosdepInternalError
    from rosdep2.lookup import RosdepLookup, ResolutionError
    from rosdep2.installers import RosdepInstaller, PackageManagerInstaller
    from rosdep2.platforms.debian import APT_INSTALLER

    rospack, rosstack = get_test_rospkgs()

    sources_loader = create_test_SourcesListLoader()
    # create our test fixture.  we want to setup a fixture that cannot resolve the rosdep data in order to test error conditions
    lookup = RosdepLookup.create_from_rospkg(rospack=rospack, rosstack=rosstack, sources_loader=sources_loader)
    context = create_default_installer_context()
    context.set_os_override('ubuntu', 'lucid')
    installer = RosdepInstaller(context)
    # - delete the apt installer
    context.set_installer(APT_INSTALLER, None)

    for verbose in [True, False]:
        resolutions, errors = lookup.resolve_all(['empty'], context)
        uninstalled = installer.get_uninstalled(resolutions, verbose)
        assert not uninstalled, uninstalled
        assert not errors

        # make sure there is an error when we lookup something that resolves to an apt depend
        resolutions, errors = lookup.resolve_all(['roscpp_fake'], context)
        uninstalled = installer.get_uninstalled(resolutions, verbose)
        assert not uninstalled, uninstalled
        assert list(errors.keys()) == ['roscpp_fake']

        resolutions, errors = lookup.resolve_all(['roscpp_fake', 'stack1_p1'], context)
        uninstalled = installer.get_uninstalled(resolutions, verbose)
        assert not uninstalled, uninstalled
        assert set(errors.keys()) == set(['roscpp_fake', 'stack1_p1'])
        print(errors)
        assert isinstance(errors['roscpp_fake'], ResolutionError), errors['roscpp_fake'][0]

    # fake/bad installer to test that we re-cast general installer issues
    class BadInstaller(PackageManagerInstaller):

        def __init__(self):
            super(BadInstaller, self).__init__(lambda x: x)

        def get_packages_to_install(*args):
            raise Exception('deadbeef')
    context.set_installer(APT_INSTALLER, BadInstaller())
    try:
        resolutions, errors = lookup.resolve_all(['roscpp_fake'], context)
        installer.get_uninstalled(resolutions)
        assert False, 'should have raised'
    except RosdepInternalError as e:
        assert 'apt' in str(e)

    # annoying mock to test generally impossible error condition
    lookup = Mock(spec=RosdepLookup)
    lookup.resolve_all.return_value = ({'bad-key': ['stuff']}, [])

    installer = RosdepInstaller(context)
    try:
        resolutions, errors = lookup.resolve_all(['roscpp_fake'], context)
        installer.get_uninstalled(resolutions)
        assert False, 'should have raised'
    except RosdepInternalError:
        pass


@contextmanager
def fakeout():
    realstdout = sys.stdout
    realstderr = sys.stderr
    fakestdout = StringIO()
    fakestderr = StringIO()
    sys.stdout = fakestdout
    sys.stderr = fakestderr
    yield fakestdout, fakestderr
    sys.stdout = realstdout
    sys.stderr = realstderr


@patch('rosdep2.installers.os.geteuid', return_value=1)
def test_RosdepInstaller_install_resolved(mock_geteuid):
    from rosdep2 import create_default_installer_context
    from rosdep2.lookup import RosdepLookup
    from rosdep2.installers import RosdepInstaller
    from rosdep2.platforms.debian import APT_INSTALLER

    rospack, rosstack = get_test_rospkgs()

    # create our test fixture.  use most of the default toolchain, but
    # replace the apt installer with one that we can have more fun
    # with.  we will do all tests with ubuntu lucid keys -- other
    # tests should cover different resolution cases.
    sources_loader = create_test_SourcesListLoader()
    lookup = RosdepLookup.create_from_rospkg(rospack=rospack, rosstack=rosstack, sources_loader=sources_loader)
    context = create_default_installer_context()
    context.set_os_override('ubuntu', 'lucid')
    installer = RosdepInstaller(context)

    with fakeout() as (stdout, stderr):
        installer.install_resolved(APT_INSTALLER, [], simulate=True, verbose=False)
    with fakeout() as (stdout, stderr):
        installer.install_resolved(APT_INSTALLER, [], simulate=True, verbose=True)
    assert stdout.getvalue().strip() == '#No packages to install'
    with fakeout() as (stdout, stderr):
        try:
            installer.install_resolved(APT_INSTALLER, ['rosdep-fake1', 'rosdep-fake2'], simulate=True, verbose=True)
        except OSError as e:
            if not any(msg in str(e) for msg in (
                '[Errno 2] No such file or directory',
                '[WinError 2] The system cannot find the file specified',
            )):
                raise
            raise SkipTest('targets ubuntu systems only')
    stdout_lines = [x.strip() for x in stdout.getvalue().split('\n') if x.strip()]
    assert len(stdout_lines) == 2
    assert stdout_lines[0] == '#[apt] Installation commands:'
    assert 'sudo -H apt-get install rosdep-fake1 rosdep-fake2' in stdout_lines, 'stdout_lines: %s' % stdout_lines



import os
from setuptools import setup

kwargs = {
    'name': 'rosdep',
    # same version as in:
    # - src/rosdep2/__init__.py
    # - stdeb.cfg
    'version': '0.22.1',
    'packages': ['rosdep2', 'rosdep2.ament_packages', 'rosdep2.platforms'],
    'package_dir': {'': 'src'},
    'install_requires': ['PyYAML >= 3.1', 'setuptools'],
    'extras_require': {
        'test': [
            'flake8',
            'flake8-comprehensions',
            "mock; python_version < '3.3'",
            'pytest',
        ],
    },
    'author': 'Tully Foote, Ken Conley',
    'author_email': 'tfoote@osrfoundation.org',
    'maintainer': 'ROS Infrastructure Team',
    'project_urls': {
        'Source code':
        'https://github.com/ros-infrastructure/rosdep',
        'Issue tracker':
        'https://github.com/ros-infrastructure/rosdep/issues',
    },
    'url': 'http://wiki.ros.org/rosdep',
    'keywords': ['ROS'],
    'entry_points': {
        'console_scripts': [
            'rosdep = rosdep2.main:rosdep_main',
            'rosdep-source = rosdep2.install:install_main'
        ]
    },
    'classifiers': [
        'Programming Language :: Python',
        'License :: OSI Approved :: BSD License'],
    'description': 'rosdep package manager abstraction tool for ROS',
    'long_description': 'Command-line tool for installing system '
                        'dependencies on a variety of platforms.',
    'license': 'BSD',
}
if 'SKIP_PYTHON_MODULES' in os.environ:
    kwargs['packages'] = []
    kwargs['package_dir'] = {}
elif 'SKIP_PYTHON_SCRIPTS' in os.environ:
    kwargs['name'] += '_modules'
    kwargs['entry_points'] = {}
else:
    kwargs['install_requires'] += ['catkin_pkg >= 0.4.0', 'rospkg >= 1.4.0', 'rosdistro >= 0.7.5', 'packaging']

setup(**kwargs)



# Copyright (c) 2012, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Dirk Thomas/dthomas@willowgarage.com

"""
API provided for rospack to determine if a dependency
is a ROSpackage or a system dependency
"""

import subprocess

from .main import _get_default_RosdepLookup
from .rospkg_loader import DEFAULT_VIEW_KEY
from .sources_list import get_sources_cache_dir


def call_pkg_config(option, pkg_name):
    try:
        value = subprocess.check_output(['pkg-config', option, pkg_name])
        return value.strip()
    except subprocess.CalledProcessError:
        return None


def init_rospack_interface():
    class Options(object):

        def __init__(self):
            self.os_override = None
            self.sources_cache_dir = get_sources_cache_dir()
            self.verbose = False
            self.dependency_types = []
    lookup = _get_default_RosdepLookup(Options())
    return lookup.get_rosdep_view(DEFAULT_VIEW_KEY)


def is_view_empty(view):
    return len(view.rosdep_defs) == 0


def is_ros_package(view, rosdep_name):
    return _ros_flag(view, rosdep_name, True)


def is_system_dependency(view, rosdep_name):
    return _ros_flag(view, rosdep_name, False)


def _ros_flag(view, rosdep_name, value):
    try:
        d = view.lookup(rosdep_name)
    except KeyError:
        return False
    ros_flag = '_is_ros' in d.data.keys()
    return ros_flag == value



# Copyright (c) 2011, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Ken Conley/kwc@willowgarage.com

import os
import traceback
from unittest.mock import patch


def get_test_dir():
    # not used yet
    return os.path.abspath(os.path.join(os.path.dirname(__file__), 'cygwin'))


def test_AptCygInstaller():
    from rosdep2.platforms.cygwin import AptCygInstaller

    @patch.object(AptCygInstaller, 'get_packages_to_install')
    def test(mock_method):
        installer = AptCygInstaller()
        mock_method.return_value = []
        assert [] == installer.get_install_command(['fake'])

        # no interactive option implemented yet
        mock_method.return_value = ['a', 'b']
        expected = [['apt-cyg', '-m', 'ftp://sourceware.org/pub/cygwinports', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=False)
        assert val == expected, val
        expected = [['apt-cyg', '-m', 'ftp://sourceware.org/pub/cygwinports', 'install', 'a', 'b']]
        val = installer.get_install_command(['whatever'], interactive=True)
        assert val == expected, val
    try:
        test()
    except AssertionError:
        traceback.print_exc()
        raise
